package com.sample;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.fail;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.DecisionTableConfiguration;
import org.drools.builder.DecisionTableInputType;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.compiler.DecisionTableFactory;
import org.drools.event.rule.AfterActivationFiredEvent;
import org.drools.event.rule.DefaultAgendaEventListener;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.FactHandle;

/**
 * 
 * Class to reproduce that in certain situations auto-focus is replaced by lock-on-active in decision table.
 * In this particular test case rule "b2" should have 'auto-focus true' and not 'lock-on-active true'.
 * 
 * @author jsvitak@redhat.com
 * @version BRMS 5.2.0 ER5
 *
 */
public class DecisionTableAutoFocus {

    public static void main(String [] args) throws Exception {
        KnowledgeBase kbase = getKnowledgeBaseFromDecisionTable("auto-focus.csv", DecisionTableInputType.CSV, true);
        StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
        OrderListener listener = new OrderListener();
        ksession.addEventListener(listener);
        
        // first test - we try to fire rule in agenda group which has auto focus disable, we won't succeed
        FactHandle withoutAutoFocus = ksession.insert("withoutAutoFocus");
        ksession.fireAllRules();
        assertEquals(listener.size(), 0);
        
        // second test - we try to fire rule in agenda group with auto focus enabled
        // it fires and it's defined consequence causes to fire second rule which has no auto focus
        ksession.insert("autoFocus");
        ksession.retract(withoutAutoFocus);
        ksession.fireAllRules();
        assertEquals(listener.size(), 2);
        String [] expected = { "b2", "b1" };
        for (int i = 0; i < listener.size(); i++)
            assertEquals(listener.get(i), expected [i]);   
    }
    
    private static KnowledgeBase getKnowledgeBaseFromDecisionTable(
            String fileName, DecisionTableInputType type, boolean printDrl) throws Exception {

        // create whole path to file
        File file = new File("test-resources", fileName);

        // decision table and its input type
        DecisionTableConfiguration dTableConfiguration = KnowledgeBuilderFactory.newDecisionTableConfiguration();
        dTableConfiguration.setInputType( type );

        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(ResourceFactory.newFileResource(file), ResourceType.DTABLE, dTableConfiguration);

        if (kbuilder.hasErrors())
        {
            System.out.println(kbuilder.getErrors());
            fail("Knowledge builder cannot compile package!");
        }
        
        // uncomment to view generated .drl file
        if (printDrl) {
            System.out.println(DecisionTableFactory.loadFromInputStream(new FileInputStream(file), dTableConfiguration));
        }

        // knowledge base
        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
        return kbase;
    }
    
    public static class OrderListener extends DefaultAgendaEventListener {
        private List<String> rulesFired = new ArrayList<String>();

        @Override
        public void afterActivationFired(AfterActivationFiredEvent event) {
            rulesFired.add(event.getActivation().getRule().getName());
        }

        public int size() {
            return rulesFired.size();
        }

        public String get(int index) {
            return rulesFired.get(index);
        }
    }
}
